initZeroPtrList(dragSource, mesh, dimensionSet(1, -2, -2, 0, 0), "dragSource");

{
    // Summed RHS of cell-based momentum balance equation
    volVectorField HSum
    (
        IOobject
        (
            "HSum",
            runTime.timeName(),
            mesh
        ),
        mesh,
        dimensionedVector(dimensionSet(1, -2, -2, 0, 0), Zero)
    );

    if (partialElimination)
    {
        forAll(movingPhases, movingPhasei)
        {
            const phaseModel& phase = movingPhases[movingPhasei];

            HSum +=
                HUEqns[phase.index()].H()
              + ddtUEqns[phase.index()].H()
              + fvOptionsEqns[phase.index()].H();
        }
    }

    forAllConstIter(phaseSystem::KdTable, fluid.Kds(), KdIter)
    {
        const volScalarField& K(*KdIter());
        const phasePair& pair(fluid.phasePairs()[KdIter.key()]);

        forAllConstIter(phasePair, pair, iter)
        {
            const phaseModel& phase = iter();
            const phaseModel& otherPhase = iter.otherPhase();

            if (phase.stationary() || otherPhase.stationary()) continue;

            if (partialElimination)
            {
                dragSource[phase.index()] += K*rAs[otherPhase.index()]*HSum;

                forAll(movingPhases, movingPhasei)
                {
                    const phaseModel& remainingPhase =
                        movingPhases[movingPhasei];

                    if
                    (
                        (remainingPhase.index() == phase.index())
                     || (remainingPhase.index() == otherPhase.index())
                    ) continue;

                    dragSource[phase.index()] -=
                    (
                        K*rAs[otherPhase.index()]
                       *(
                            HUEqns[remainingPhase.index()].A()
                          + ddtUEqns[remainingPhase.index()].A()
                          + fvOptionsEqns[remainingPhase.index()].A()
                        )
                       *phases[remainingPhase.index()].U()
                    );
                }
            }
            else
            {
                dragSource[phase.index()] += K*phases[otherPhase.index()].U();
            }
        }
    }
}
